import $ from 'jquery'
import { Plugin } from 'foundation-sites/js/foundation.core.plugin'
import { debounce } from '@moreonion/js-utils/src/utils'

/**
 * AdaptiveSticky module.
 *
 * @module foundation.adaptiveSticky
 */

class StickyButton extends Plugin {
  /**
   * Creates a new instance.
   * @class
   * @name StickyButton
   * @fires StickyButton#init
   * @param {Object} element - jQuery object to add the trigger to.
   * @param {Object} options - Overrides to the default plugin settings.
   */
  _setup(element, options = {}) {
    this.$element = element
    this.className = 'StickyButton'
    // defaults, fallbacks
    this.options = $.extend(true, {}, StickyButton.defaults, this.$element.data(), options)
    this.$showAfter = $(this.options.after)

    this._log('setup', this)
    this._init()
  }

  /**
   * Initializes the plugin and calls functions to get functioning on load.
   * @function
   * @private
   */
  _init() {
    // Show (or hide) sticky buttons.
    if (this.$showAfter.length) {
      $(window).on('scroll.zf.stickyButton resize.zf.stickyButton', debounce(() => {
        this._hasPassed(this.$showAfter) ? this._show() : this._hide()
      }, 100))
    } else {
      $(window).on('resize.zf.stickyButton', debounce(() => this._show(), 100))
    }
    this.$element.find('a').on('click', () => this._hide())
    this._log('init', this)
  }

  /**
   * Check if we’ve (vertically) scrolled down past an element.
   * @function
   * @param {jQuery} $element The element to check.
   * @return {boolean} Whether scrollTop is below the element.
   * @private
   */
  _hasPassed ($element) {
    const elementBottom = $element.offset().top + $element.outerHeight()
    this._log('scrolled passed', $element)
    return elementBottom < $(window).scrollTop()
  }

  /**
   * Reveal the sticky button.
   * @function
   * @fires stickyButton#show
   * @fires stickyButton#shown
   * @private
   */
  _show () {
    this._log('show', this.$element)
    /**
     * Fires when the showing starts.
     * @event stickyButton#show
     */
    this.$element.trigger('show.zf.stickyButton')
    this.$element.filter('.top').show()
      // Prevent the element from taking up space in the DOM.
      .css('margin-bottom', -1 * this.$element.filter('.top').outerHeight())
      .css('top', 0)
    this.$element.filter('.bottom').show().css('bottom', 0)
    /**
     * Fires when showing is finished.
     * @event stickyButton#shown
     */
    this.$element.trigger('shown.zf.stickyButton')
  }

  /**
   * Hide the sticky button.
   * @function
   * @fires stickyButton#hide
   * @fires stickyButton#hidden
   * @private
   */
  _hide () {
    this._log('hide', this.$element)
    /**
     * Fires when the hiding starts.
     * @event stickyButton#hide
     */
    this.$element.trigger('hide.zf.stickyButton')
    this.$element.filter('.top').css('top', -1 * this.$element.filter('.top').outerHeight())
    this.$element.filter('.bottom').css('bottom', -1 * this.$element.filter('.bottom').outerHeight())
    // Wait for CSS transitions to finish before hiding the element.
    if (this.$element.css('transition')) {
      this.$element.on('transitionend.zf.stickyButton', () => {
        this.$element.hide().off('transitionend.zf.stickyButton')
      })
    } else {
      this.$element.hide()
    }

    /**
     * Fires when the hiding is finished.
     * @event stickyButton#hidden
     */
    this.$element.trigger('hidden.zf.stickyButton')
  }

  /**
   * Logs debug output to the console.
   * @function
   * @private
   */
  _log () {
    if (this.options.debug) {
      console.debug('[StickyButton]', ...arguments)
    }
  }

  /**
   * Destroys an instance.
   * @function
   */
  _destroy() {
    $(window).off('.zf.stickyButton')
    this._hide()
    this._log('destroyed', this)
  }
}

/**
 * Default settings for plugin.
 */
StickyButton.defaults = {
  /**
   * The selector of an element after which the sticky button is shown.
   * @option
   * @type {string|null}
   * @default 'null'
   */
  after: null,

  /**
   * Toggle debug output.
   * @option
   * @type {boolean}
   * @default false
   */
  debug: false
}

export { StickyButton }
